;
; chkin.asm - CHKIN kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Chkin (X=logical file numbe)
;
; Used registers: A X (Y preserved)
; Status: C
; C=1:
; A=3 file not open
; A=5 device not present
; A=6 (not input file)
; Status response:
; ST=$00 if no error
; ST=$03 timeout
; ST=$80 if device not present
; Secondary address 128..255 then communication on serial bus is a bit different...
;**************************************************
newchkin	.proc
		cpx #0
		beq chkin_nop
		jsr search_logical_X_st
		beq chkin_cont1
chkin_nop	jmp exit_file_not_open

chkin_cont1	jsr readfiletables
		jsr search_buffer_A
		beq chkin_cont3		;Ok found buffer for file

		#ldatax open_files_currentp_exp
		lda open_files_info_exp,x
		ldx #MISCIN
		sta buffer_info_exp,x
		cmp #INFO_KERNAL
		bne chkin_cont2

		lda ActualDev
		sta buffer_drive_exp,x
		bne *+4			;keyboard?
		ldx #STDIN
		#stx_l_ buffer_input_exp

		cmp #4
		bcs cw			;serial device?
		#psr $f21b
		lda ActualDev
		jmp exit2basic

cw		ldx #3
cy		cmp current_floppy8_exp,x
		bne xg
		txa
		ora #8
		ldx #0
xg		dex
		bpl cy

		tax
		jsr OP_cfg16up
		txa
		jsr iectalk
		lda ActualSecAddr
		bpl sfc
		jsr $edcc		;wait 1ms
		jmp *+6
sfc		jsr iectksa
		jsr OP_cfg16down
		txa
		bit Status
		bmi exit_device_not_present
		sta ActualInput
		clc
		jmp exit2basic		;to $f21b

chkin_cont2	lda ActualDev
		sta buffer_drive_exp,x

chkin_cont3	.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		jsr chkin_doch
		.else
		#stx_l_a buffer_input_exp
		.fi

		.if INPUT_OUTPUT_COMPATIBILITY=1
		lda #8
		sta ActualInput		;Lamer
		.fi

		clc
		jmp exit2basic
		.pend

		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
chkin_doch	.proc
		php
		pha
		txa
		sta buffer_input_exp
		clc
		adc #<buffer_info_exp
		sta newchrin_ram_at_exp
		sta newgetin_ram_at_exp
		txa
		clc
		adc #<buffer_address_low_exp
		sta newchrin_ram_at2_exp
		txa
		clc
		adc #<buffer_address_hi_exp
		sta newchrin_ram_at3_exp
		pla
		plp
		rts
		.pend
		.fi
