;
; close.asm - CLOSE kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Close (A=logical file number)
;
; Used registers: A X (Y preserved only if closeing recently opened file. This is used in some buggy software...)
; Status: C
; C=1:
; Status response:
; ST=$00 if no error
; ST=$03 timeout
; ST=$80 if device not present
; Notes:
; Secondary address 128..255 then no action is taken
;**************************************************
newclose	.proc
		cmp #0
		beq close_nop
		jsr search_logical_A
		beq close_ano1
close_nop	clc			;File already closed
		jmp exit2basic

close_ano1	jsr readfiletables
		jsr search_buffer_A
		beq close_ano2		;Ok found buffer for file

		#ldatax open_files_currentp_exp
		lda open_files_info_exp,x
		cmp #INFO_KERNAL
		bne close_ano3

		jsr close_real

		lda ActualDev
		ldx #3
cy		cmp current_floppy8_exp,x
		bne xg
		txa
		ora #8
		ldx #0
xg		dex
		bpl cy

		tax
		beq gf			;keyboard
		cmp #3
		beq gf			;screen
		bcc thui		;tape, etc
		lda ActualSecAddr
		bmi gf

		#psr $f648
		txa
gf		clc
		jmp exit2basic		;Do the kernal stuff...

thui		lda #1
		sta OpenFiles		;Fake kernal about files table
		lda #0
		pha
		#psr $f2a7
		txa
		jmp exit2basic		;Do the kernal stuff...

close_real	lda ActualLogical
		#ldx_l_ buffer_input_exp
		jsr closekill_inout	;Disconnect I/O from closed file
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		jsr chkin_doch
		.else
		#stx_l_ buffer_input_exp
		.fi
		#ldx_l_ buffer_output_exp
		jsr closekill_inout
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		jsr chkout_doch
		.else
		#stx_l_ buffer_output_exp
		.fi

		#ldatax open_files_currentp_exp
		lda #0
		sta open_files_logical_exp,x;Now file is closed
		#ldx_l_ buffer_currentp_exp
		sta buffer_logical_exp,x;Now file buffer is freed
		rts

closekill_inout	cmp buffer_logical_exp,x
		bne close_an1
		ldx #IGNOREINOUT
close_an1	rts

close_ano2	#phy_a
		jsr OP_save_zeropage
		jsr XIT_close
		jsr OP_restore_zeropage
		#ply_a

close_ano3
		jsr close_real
		clc
		jmp exit2basic

XIT_close	#xit_init

		#ldatax buffer_currentp_exp
		lda buffer_drive_exp,x
		sta current_drive_exp
		#ldy_l_x_a buffer_info_exp
		cpy #INFO_STATUS
		beq OP_XIT_do_command_channel
		lda buffer_type_exp,x
		cmp #BUFFERMODE_FILE
		beq XIT_do_file
cl2		rts

XIT_do_file	lda #INFO_IGNORE	;be safe
		sta buffer_info_exp,x

		cpy #INFO_BLK_RW
		beq bf
		cpy #INFO_BLK_RW_DIRTY
		beq cl
		cpy #INFO_BLK_RW_EOF
		beq bf
		cpy #INFO_BLK_RW_EOF_DIRTY
		beq cl
		cpy #INFO_BLK_WRITE
		beq bf
		cpy #INFO_BLK_WRITE_DIRTY
		bne cl2

cl		jsr update_filelen
		jsr XIT_flush_buf
		#jmps ko

bf		jsr update_filelen
ko		jsr restore_allocator_state

		jsr XIT_find_free_flush

		jsr OP_load_next_sector_treep;restore tree sector
		lda buffer_start,x
		ora #>b_cfs_file_tree_exp;write out the tree sector
		jsr XIT_write_a_sector	;-=> error

		lda #>b_cfs_file_misc_exp
		jsr buffer_subpart_cp
		ldy #<b_cfs_file_dirsect_exp
		ldx #3
lc		#lda_zp_y current_path
		sta next_sector,x
		iny
		dex
		bpl lc

		lda #ram_exp_buffer0
		jsr XIT_read_a_sector	;-=> error

		ldy #<b_cfs_file_dir_offs_exp
		#lda_zp_y current_path
		asl
		sta current_path
		lda #ram_exp_buffer0
		adc #0
		sta current_path+1

		ldy #$10		;filesize offset
		#ldatax buffer_currentp_exp
		lda buffer_filesize1_exp,x
		#sta_zp_y current_path
		iny
		lda buffer_filesize2_exp,x
		#sta_zp_y current_path
		iny
		lda buffer_filesize3_exp,x
		#sta_zp_y current_path
		iny
		lda buffer_filesize4_exp,x
		#sta_zp_y current_path

		lda #ram_exp_buffer0	;write out the final directory entry
		jmp XIT_write_a_sector	;-=> error
		.pend
