;
; clrchn.asm - CLRCHN, CLALL kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Clall & Clrchn
;
; Used registers: A X (Y preserved, A=0!!!)
; Status: C
; C=1:
; A=5 device not present
; Status response:
; ST=$00 if no error
; ST=$03 timeout
; ST=$80 if device not present
;**************************************************

newclall	.proc
		ldx #MAX_OPEN_FILES
		lda #0
qho		sta open_files_logical_exp-1,x;Kernal does this, however
		dex			;closing files would be better
		bne qho
		ldx #MAX_BUFFERS
qho2		sta buffer_logical_exp-1,x;Kernal does this, however
		dex			;closing files would be better
		bne qho2

newclrchn	.proc
		#ldatax buffer_output_exp
		sta buffer_currentp_exp	;OP_do_command_channel
		lda buffer_info_exp,x
		cmp #INFO_STATUS
		bne clrchn_ano1
		#phy_a
		jsr OP_save_zeropage
		jsr OP_XIT_do_command_channel;IDE64 command_channel?
		jsr OP_restore_zeropage
		#ply_a
		#jmps clrchn_ano2

clrchn_restore	.global
		#psr switch_16k
		#ldatax buffer_output_exp
		lda buffer_info_exp,x

clrchn_ano1	cpx #STDOUT
		beq clrchn_ano2
		cmp #INFO_KERNAL
		bne clrchn_ano2
		lda #3
		cmp buffer_drive_exp,x
		bcs clrchn_ano2
		.if INPUT_OUTPUT_COMPATIBILITY=0
		sta ActualOutput
		.fi

		lda #STDOUT
		sta buffer_output_exp	;to fix unlisten loop
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		lda #<(buffer_info_exp+STDOUT)
		sta newchrout_ram_at_exp
		.fi

		jsr OP_cfg16up
		jsr iecunlisten
		jsr OP_cfg16down

clrchn_ano2	lda #STDOUT
		sta buffer_output_exp
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		lda #<(buffer_info_exp+STDOUT)
		sta newchrout_ram_at_exp
		.fi

		.if INPUT_OUTPUT_COMPATIBILITY=1
		lda #0
		sta ActualInput		;Lamer
		lda #3
		sta ActualOutput	;Lamer
		.fi

		#ldatax buffer_input_exp
		cpx #STDIN
		beq clrchn_ano3
		lda buffer_info_exp,x
		cmp #INFO_KERNAL
		bne clrchn_ano3
		lda #3
		cmp buffer_drive_exp,x
		bcs clrchn_ano3x
		.if INPUT_OUTPUT_COMPATIBILITY=0
		lda #0
		sta ActualInput
		.fi

		lda #STDIN
		sta buffer_input_exp	;to fix untalk loop
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		lda #<(buffer_info_exp+STDIN)
		sta newchrin_ram_at_exp
		sta newgetin_ram_at_exp
		.fi

		jsr OP_cfg16up
		jsr iecuntalk
		jsr OP_cfg16down
clrchn_ano3	clc
clrchn_ano3x
		lda #STDIN
		sta buffer_input_exp
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		lda #<(buffer_info_exp+STDIN)
		sta newchrin_ram_at_exp
		sta newgetin_ram_at_exp
		.fi

		lda #0			;Lame kernal (see $f6f4...)
		jmp exit2basic
		.pend
