;
; dir.asm - directory handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

XIT_dir_first	.proc
		pha
		lda path_exp+1		;"$0" hack...
		eor #"0"
		ora path_exp+2
		bne dollar0
		sta path_exp+1
dollar0

kojq		jsr OP_short_path
		cmp #"="
		bne nospecial
		jsr OP_short_path
		cmp #"P"
		bne kojq
		jsr OP_short_path
		jsr OP_get_current_drive_type
		and #FS_MASK
		eor #FS_CFS
		cmp #1
		jsr LWOP_reporterror_71_xit;-=> partition listing only for CFS!

		lda #DIRMODE_PARTITION_START
		sta direntry_mode_exp
		ldy #0
		sty next_sector
		sty next_sector+1
		sty next_sector+2
		iny
		sty next_sector+3
		lda #255
		gne cke
nospecial
		lda #RESOLV_LAST_REMAINS
		jsr OP_XIT_resolv_path_part;-=> error
		lda path_partition_exp
		bne cke
		#ldatax current_drive_exp
		lda drive_workpartition_exp,x
		clc
		adc #1
cke		sta direntry_partition_exp
		lda #0
		sta direntry_used_exp
		sta direntry_used_exp+1
		pla
		sta direntry_first_exp

		ldy #"="
		jsr OP_path_searchtype
		bcc sf
		lda #"*"
		sta path_searchtype_exp
		lda #0
		sta path_searchtype_exp+1
sf
		lda path_exp		;Empty="*"
		bne tko
		sta path_exp+1
		lda #"*"
		sta path_exp
tko					;set up wildcard optimization
		ldx #0
		lda path_exp
		eor #"*"
		ora path_exp+1
		bne *+4
		ldx #128		;no name wildcard
		lda path_searchtype_exp
		eor #"*"
		ora path_searchtype_exp+1
		bne *+3
		inx			;no type wildcard
		#stx_l_a direntry_wildcard_exp
		rts
		.pend
;------------
XIT_dirlp2	.proc
		jsr XIT_upload_next_sector;-=> error
		ldy #1			;1 sector
		jsr OP_read
		jsr LWOP_reporterror_read_xit;-=> error

		lda #15			;16 directory entrys
		sta direntry_number_exp
dirlp		jsr dirnotlast
		bcs jw
		txa
		adc start_zeropage+5
		sta start_zeropage+5
		bcc *+4
		inc start_zeropage+6
jw
		#dec_l_a direntry_number_exp
		bpl dirlp

		jsr ata_pi_nomore_data
		jsr LWOP_reporterror_read_xit;-=> error

		lda next_sector
		ora next_sector+1
		ora next_sector+2
		ora next_sector+3
		bne bz

		jsr kvq
		lda #32
		clc
		adc start_zeropage+5
		sta start_zeropage+5
		bcc *+4
		inc start_zeropage+6
		rts

kvq		lda #255
		sta direntry_first_exp	;end of directory

		lda #1
		pha			;1
		pha			;2

		lda direntry_used_exp
		pha			;3
		lda direntry_used_exp+1
		pha			;4
		lda direntry_mode_exp
		eor #DIRMODE_PARTITION_END
		cmp #1
		ldx #12
dfsvcxc		lda blkused_text,x	;17
		bcs vd
		lda prtused_text,x
vd		pha
		dex
		bpl dfsvcxc
		ldx #12
gv		pha			;29
		dex
		bne gv
		txa
		pha			;30
		pha			;31
		pha			;32
		ldy #31
		jmp start_zeropage

bz		rts

dirnotlast	jsr OP_read_to_direntry

		lda direntry_first_exp
		beq notdirlabel

		lda direntry_first_exp
		ldy #29
		lsr
		beq hnuhi		;load, skip $0401
					;lda #1
		pha			;1
		lda #4
		pha			;2
		ldy #31

hnuhi		lda direntry_attrib_exp
		and #%10000111
		eor #FTYPE_DIR
		cmp #1
		jsr LWOP_reporterror_71_xit;-=> directory error

		lda #1
		pha			;3
		pha			;4

		lda direntry_partition_exp
		pha			;5 currentpartition number
		lda #0
		pha			;6
		sta direntry_first_exp
		#psr $1223		;7,8 rvson"
		ldx #240

dcb		lda direntry_name_exp-240,x
		pha			;24
		inx
		bne dcb
		ldx #7
dfsvcx		lda dirlabel_text,x
		pha			;32
		dex
		bpl dfsvcx
		tya
		tax
		inx
		clc
		jmp start_zeropage

nomatch		sec
		rts

notdirlabel	lda direntry_hidden_exp
		bmi nomatch		;hidden entry...
		lda direntry_attrib_exp
		and #FATTR_CLOSED | FTYPE_MASK
		beq nomatch		;free entry...
		and #FTYPE_MASK
		eor #FTYPE_RELATIVE
		bne uie			;to report correct filesize
		sta direntry_recordsize_exp;for relative files...
uie		lda direntry_wildcard_exp
		bmi dirskip1

		jsr check_wildcards
		bcs nomatch

		lda direntry_wildcard_exp
dirskip1	lsr
		bcs dirskip2

		inc current_path+1
		jsr check_wildtype
		dec current_path+1
		bcs nomatch

dirskip2	tsx
		txa
		sec
		sbc #31
		sta direntry_spc_exp

		lda #1
		pha			;1
		pha			;2

		.if DIRECTORY_BLOCKSIZE_256=1
		lda direntry_size_exp
		clc
		adc #<255
		lda direntry_size_exp+1
		adc #>255
		tay
		lda direntry_size_exp+2
		adc #0
		tax
		lda direntry_size_exp+3
		adc #0
		beq *+6
		ldx #255
		ldy #255
		tya
		.else
		lda direntry_size_exp
		clc
		adc #<511
		lda direntry_size_exp+1
		adc #>511
		tay
		lda direntry_size_exp+2
		adc #0
		tax
		lda direntry_size_exp+3
		adc #0
		lsr a
		beq *+6
		ldx #255
		ldy #255
		txa
		ror a
		tax
		tya
		ror a
		tay
		.fi
		pha			;3
		lda direntry_mode_exp
		cmp #DIRMODE_PARTITION_END
		bne *+4
		ldy #1
		tya
		clc
		adc direntry_used_exp
		sta direntry_used_exp
		txa
		adc direntry_used_exp+1
		bcc noovf		;In case of overflow 65535
		lda #255
		sta direntry_used_exp
noovf		sta direntry_used_exp+1

		txa
		pha			;4
		cpy #<10
		sbc #>10
		bcc sds
		txa
		cpy #<100
		sbc #>100
		bcc sds2
		txa
		cpy #<1000
		sbc #>1000
		bcc sds3
		txa
		cpy #<10000
		sbc #>10000
		bcc sds5
		lda #$14
		gcs sds4

sds		lda #$20
		pha			;x
sds2		lda #$20
		pha			;x
sds3		lda #$20
sds4		pha			;x

sds5		lda #$22
		pha
		ldx #240

ncvbx		lda direntry_name_exp-240,x
		beq ncvbx2
		pha			;x
		inx
		bne ncvbx
		lda #$22
		pha
		gne bde
ncvbx2		lda #$22
		pha
		lda #$20
ncvbx4		pha			;x
		inx
		bmi ncvbx4

bde		lda direntry_attrib_exp	;closed?
		asl
		lda #$20
		bcs *+4
		lda #$2a		;*
		pha			;x
		ldx #253

cbe		lda direntry_filetype_exp-253,x
		beq dirdata
		pha			;x
		inx
		bne cbe
dirdata		lda direntry_attrib_exp
		and #%01010000
		cmp #%01010000		;Deleteable & writeable?
		lda #"<"
		bcc *+4
		lda #$20
		pha

jrr		lda #$20
jr		pha			;x
		.if SUPERCPU=1
		tsc
		cmp direntry_spc_exp
		bne jrr
		.else
		tsx
		cpx direntry_spc_exp
		bne jr
		.fi
		lda #0
		pha			;x EOL
		ldy #31
		ldx #32
		clc
		jmp start_zeropage

dirlabel_text	.byte $00,$34,$36,$45,$44,$49,$20,$22;" IDE64
blkused_text	.byte $20,".","D","E","S","U",$20,"S","K","C","O","L","B"
prtused_text	.byte $20,$20,".","S","N","O","I","T","I","T","R","A","P"
		.pend

;---------------------------------------
;Read 32 bytes to direntry_exp C=1 if error
;---------------------------------------
OP_read_to_direntry .proc
		#ldatax direntry_mode_exp
		bne ofs
		ldx #224
cb2		lda data_g		;DATA PORT
		lda low_reg		;0-7 BIT
		sta direntry_exp-224,x
		inx
		lda hig_reg		;8-15 BIT
		sta direntry_exp-224,x
		inx
		bne cb2
		lda direntry_nexts_exp
		asl
		asl
		asl
		rol next_sector
		rol next_sector+1
		rol next_sector+2
		rol next_sector+3
		asl
		rol next_sector
		rol next_sector+1
		rol next_sector+2
		rol next_sector+3
		rts
					;Hack to read old filesystem

ofs		dex
		bne partition
		lda direntry_number_exp
		bne kqj
		sta direntry_attrib_exp

		ldx #12			;Skip until end of sector
skip		bit data_g
		dex
		bne skip
		rts

t1		.text "DSPURCDU",0,"C"
t2		.text "EERSEBIN",0,"F"
t3		.text "LQGRLMRK",0,"S"
t4		.byte FTYPE_DEL,FTYPE_NORMAL,FATTR_EXEC | FTYPE_NORMAL,FTYPE_NORMAL
		.byte FTYPE_NORMAL,FTYPE_NORMAL,FATTR_EXEC | FTYPE_DIR,FTYPE_NORMAL

kqj		eor #15
		bne nueu		;Not first

		bit data_g		;($00)
		bit low_reg		;last sector?
		bvc uxjk
		sta next_sector
		sta next_sector+1
		sta next_sector+2
		sta next_sector+3
		ldx 3
		gne uue

uxjk		lda low_reg
		and #%00111111
		sta next_sector+1
		lda hig_reg
		sta next_sector+2
		lda data_g		;($02)
		lda low_reg
		and #15
		sta next_sector
		lda hig_reg
		sta next_sector+3

		ldx #2			;($04)
uue		jsr skip		;($06)

nueu		lda data_g		;Attrib ($00)
		#ldatax low_reg
		and #3
		tay
		txa
		and #%00010000
		beq *+4			;DIR?
		ldy #6
		jsr fillftype
		txa
		pha
		and #%11000000
		eor #%01100000
		asl
		bpl *+4			;Write protect?
		ora #%00100000
		ror
		ora t4,y
		cpy #6			;DIR?
		bne ded3
		ora #0
		bmi ded3
		lda #0			;*DIR????
ded3
		sta direntry_attrib_exp

		lda #0			;TODO: convert date
		sta direntry_date_exp
		sta direntry_date_exp+1
		sta direntry_date_exp+2
		sta direntry_date_exp+3

		jsr getname

		pla
		cmp #$f0
		bne tre2
		lda direntry_attrib_exp	;Dirlabel needs padding...
		and #%01111111
		sta direntry_attrib_exp
		gcs tre
					;Kill $a0 padding of name
tre2		ldx #15
ktt		lda direntry_name_exp,x
		eor #$a0
		bne tre
		sta direntry_name_exp,x
		dex
		bpl ktt
tre
		ldx #0
		jsr getsize		;($12)
		ldx #2
		jsr getsize		;($14)
		inx
		jsr skip
		lda data_g		;($1c)
		lda low_reg
		sta direntry_pointer_exp+1
		lda hig_reg
		sta direntry_pointer_exp+2
		lda data_g		;($1e)
		lda low_reg
		and #15
		sta direntry_pointer_exp
		lda hig_reg
		sta direntry_pointer_exp+3
		rts

fillftype	lda t1,y
		sta direntry_filetype_exp
		lda t2,y
		sta direntry_filetype_exp+1
		lda t3,y
		sta direntry_filetype_exp+2
		rts

getsize		lda data_g		;($12)
		lda low_reg
		sta direntry_size_exp,x
		lda hig_reg
		sta direntry_size_exp+1,x
		rts

getname		ldx #240
cb23		lda data_g		;Get name ($02)
		lda low_reg
		sta direntry_name_exp-240,x
		inx
		lda hig_reg
		sta direntry_name_exp-240,x
		inx
		bne cb23
		rts

partition	dex
		bne partition2
		#stx_l_a direntry_attrib_exp
		lda direntry_number_exp
		bne be
		lda #DIRMODE_PARTITION_END
		sta direntry_mode_exp
en		rts
be		eor #15
		bne en
		bit data_g
		bit data_g
		lda hig_reg
		sta direntry_defpart_exp
		ldx #10
		jsr skip
		ldx #0
		jsr getnextsector
		ldx #2
		jsr getnextsector
		jsr skip
		jsr getname
		lda #FTYPE_DIR
		sta direntry_attrib_exp
		ldx #232
		gne skip

getnextsector	lda data_g
		lda low_reg
		sta next_sector,x
		lda hig_reg
		sta next_sector+1,x
		rts

partition2	lda #0
		ldx #3
vkf		sta next_sector,x
		sta direntry_size_exp,x
		dex
		bpl vkf
		sta direntry_pointer_exp
		sta direntry_filetype_exp
		lda #16
		sec
		sbc direntry_number_exp
		.if DIRECTORY_BLOCKSIZE_256=0
		asl
		.fi
		sta direntry_size_exp+1
		jsr getname
		lda direntry_number_exp
		eor #15
		eor direntry_defpart_exp
		cmp #1
		bit data_g
		lda low_reg
		bpl nn			;not valid
		and #%00110000
		beq nwrite
		and #%00100000
		bne nn			;hidden
		lda #(FTYPE_NORMAL|FATTR_DELETE|FATTR_WRITE|FATTR_READ|FATTR_EXEC) << 1
		.byte SKIP2
nwrite		lda #(FTYPE_NORMAL|FATTR_DELETE|FATTR_READ|FATTR_EXEC) << 1
		ror
		.byte SKIP2
nn		lda #0
		sta direntry_attrib_exp
		bit data_g
		bit data_g
		lda low_reg
		ldy #8
		and #%10110000
		beq bd			;unformatted
		cmp #%00010000
		beq bd2			;cfs
		dey			;unknown
		.byte SKIP1
bd2		iny
bd		jsr fillftype
		ldx #5
		gne skip
		.pend
