;
; messages.asm - error channel message handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

invalidate_drive .proc
		jsr OP_get_current_drive_type
		and #<~FS_MASK
		sta drive_type_exp,x	;Unknown filesystem!

		ldx #MAX_BUFFERS-1
qho2		lda buffer_logical_exp,x;buffer used?
		beq x
		lda buffer_drive_exp,x	;this drive?
		eor current_drive_exp
		bne x
		lda buffer_info_exp,x
		eor #INFO_STATUS
		beq x			;status channel skipped
		lda #INFO_IGNORE	;file invalidated
		sta buffer_info_exp,x
x		dex
		bpl qho2
		rts
		.pend

LWOP_reporterror_xit .proc
		bcc ok
		cmp #ETIMEOUT
		beq LWOP_reporterror_21_xit
		cmp #EUNEXP
		beq LWOP_reporterror_74_xit
		cmp #EATAERROR
		beq th
		cmp #EATAPIERROR
		bne ok
		bit error_g
		lda low_reg
		cmp #ATAPI_SENSE_UNIT_ATTENTION
		beq LWOP_reporterror_29_xit

th		sec
		bcs LWOP_reporterror_31_xit
ok		clc
		rts
		.pend

LWOP_reporterror_read_xit .proc
		bcc ok
		cmp #ETIMEOUT
		beq LWOP_reporterror_21_xit
		cmp #EUNEXP
		beq LWOP_reporterror_74_xit
		bit error_g
		cmp #EATAERROR
		bne nata
		ldx #23
		lda low_reg
nk		asl
		bcs b
		dex
		cmp #16
		bne nk
		geq b

nata		cmp #EATAPIERROR
		bne ok
		lda low_reg
		lsr
		lsr
		lsr
		lsr
		tax
b		lda error_trans,x
		beq ok
		bpl LWOP_normerr
		gmi LWOP_nullerr

ok		clc
		rts

error_trans	.byte 0			;no sense
		.byte 0			;recovered error
		.byte 74+$80		;not ready
		.byte 23		;medium error
		.byte 21		;hardware error
		.byte 67		;illegal request
		.byte 29+$80		;unit attention
		.byte 26+$80		;data protect
		.byte 22		;blank check
		.byte 20		;vendor specific
		.byte 20		;copy aborted
		.byte 74+$80		;aborted command
		.byte 20		;obsolote
		.byte 67		;volume overflow
		.byte 23		;miscompare
		.byte 20		;reserved

		.byte 22		;amnf
		.byte 67		;tk0nf
		.byte 23		;abrt
		.byte 29+$80		;mcr ???
		.byte 67		;idnf
		.byte 29+$80		;mc
		.byte 20		;unc
		.byte 23		;bbk
		.pend

LWOP_reporterror_write_xit .proc
		bcc ok
		cmp #ETIMEOUT
		beq LWOP_reporterror_28_xit
		cmp #EUNEXP
		beq LWOP_reporterror_74_xit
		bit error_g
		cmp #EATAERROR
		bne nata
		ldx #23
		lda low_reg
nk		asl
		bcs b
		dex
		cmp #16
		bne nk
		geq b

nata		cmp #EATAPIERROR
		bne ok
		lda low_reg
		lsr
		lsr
		lsr
		lsr
		tax
b		lda error_trans,x
		beq ok
		bpl LWOP_normerr
		gmi LWOP_nullerr

ok		clc
		rts

error_trans	.byte 0			;no sense
		.byte 0			;recovered error
		.byte 74+$80		;not ready
		.byte 25		;medium error
		.byte 28		;hardware error
		.byte 67		;illegal request
		.byte 29+$80		;unit attention
		.byte 26+$80		;data protect
		.byte 22		;blank check
		.byte 20		;vendor specific
		.byte 20		;copy aborted
		.byte 74+$80		;aborted command
		.byte 20		;obsolote
		.byte 67		;volume overflow
		.byte 25		;miscompare
		.byte 20		;reserved

		.byte 22		;amnf
		.byte 67		;tk0nf
		.byte 25		;abrt
		.byte 29+$80		;mcr ???
		.byte 67		;idnf
		.byte 29+$80		;mc
		.byte 20		;unc
		.byte 25		;bbk
		.pend

LWOP_reporterror_26_xit .proc
		lda #$80+26
		.byte SKIP2
		.pend
LWOP_reporterror_28_xit .proc
		lda #$80+28
		.byte SKIP2
		.pend
LWOP_reporterror_29_xit .proc
		lda #$80+29
		.byte SKIP2
		.pend
LWOP_reporterror_30_xit .proc
		lda #$80+30
		.byte SKIP2
		.pend
LWOP_reporterror_31_xit .proc
		lda #$80+31
		.byte SKIP2
		.pend
LWOP_reporterror_32_xit .proc
		lda #$80+32
		.byte SKIP2
		.pend
LWOP_reporterror_33_xit .proc
		lda #$80+33
		.byte SKIP2
		.pend
LWOP_reporterror_34_xit .proc
		lda #$80+34
		.byte SKIP2
		.pend
LWOP_reporterror_39_xit .proc
		lda #$80+39
		.byte SKIP2
		.pend
LWOP_reporterror_50_xit .proc
		lda #$80+50
		.byte SKIP2
		.pend
LWOP_reporterror_62_xit .proc
		lda #$80+62
		.byte SKIP2
		.pend
LWOP_reporterror_63_xit .proc
		lda #$80+63
		.byte SKIP2
		.pend
LWOP_reporterror_64_xit .proc
		lda #$80+64
		.byte SKIP2
		.pend
LWOP_reporterror_70_xit .proc
		lda #$80+70
		.byte SKIP2
		.pend
LWOP_reporterror_74_xit .proc
		lda #$80+74
		.byte SKIP2
		.pend
LWOP_reporterror_75_xit lda #$80+75
		bcc noer
LWOP_nullerr	#ldx_l_ current_drive_exp
		sta status_pos_exp,x
		cmp #29+$80
		bne piei
		jsr invalidate_drive
piei		#xit_exit
		lda xit_bk_exp

		.if BANK=0
		lsr
		bcs noer
		lsr
		bcs exitbank1
		lsr
		bne exitbank3
		geq exitbank2
		.fi

		.if BANK=1
		lsr
		bcs exitbank0
		lsr
		bcs noer
		lsr
		beq exitbank2
		gne exitbank3
		.fi

		.if BANK=2
		lsr
		bcs exitbank0
		lsr
		bcs exitbank1
		lsr
		bne exitbank3
		.fi

		.if BANK=3
		lsr
		bcs exitbank0
		lsr
		bcs exitbank1
		lsr
		beq exitbank2
		.fi
noer		rts

LWOP_reporterror_ok .proc
		#ldatax current_drive_exp
		lda #$80
		sta status_pos_exp,x
		rts
		.pend

LWOP_reporterror_71_xit .proc
		lda #71
		.byte SKIP2
		.pend
LWOP_reporterror_21_xit
		lda #21
		bcc noer
LWOP_normerr	jsr LWOP_disk_status
		lda head_g
		lda low_reg
		and #15
		jsr append_status_dec
		lda cylhi_g
		lda low_reg
		jsr append_status_dec
		lda cyllow_g
		lda low_reg
		jsr append_status_dec
		lda sector_g
		lda low_reg
vxfd		jsr append_status_dec
		lda #0
		sta drive_status_exp,x
		geq piei

LWOP_reporterror_01_xit .proc
		lda #1
		.byte SKIP2
		.pend
LWOP_reporterror_02_xit .proc
		lda #2
		.byte SKIP2
		.pend
LWOP_reporterror_73_xit .proc
		lda #73
		.byte SKIP2
		.pend
LWOP_reporterror_76_xit .proc
		lda #76
		.byte SKIP2
		.pend
LWOP_reporterror_77_xit
		lda #77
		.if SUPERCPU=1
		phy
		phx
		.else
		sta pom1
		tya
		pha
		txa
		pha
		lda pom1
		.fi
		jsr LWOP_disk_status
		pla
		jsr append_status_dec
		pla
		jsr append_status_dec
		lda #0
		jsr append_status_dec
		lda #0
		geq vxfd

