;
; path.asm - path resolver
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;----------------------------------
;Get filename to path_exp
;
OP_copy_filename_to_path .proc
		jsr OP_cfg16up
		lda #0
		ldy ActualFNLength
		ldx ActualFNLength
		#jmps twb
xip		dey
		#sta_cfg_16
		lda (ActualFilename),y
		dex
twb		#sta_cfg_open
		sta path_exp,x
		bne xip
		jmp OP_cfg16down
		.pend
;----------------------------------
;Search for ":", if found C=1
;
		#opencfg_start
OP_delimiter_in_path .proc
		ldx #0
gcrp		lda path_exp,x
		beq ntq2
		cmp #":"
		beq ntq
		inx
		bne gcrp
ntq2		clc
ntq		rts
		.pend
		#opencfg_end
;----------------------------------
; cut off the beginning of path
;
OP_short_path_here .proc
		#phx_a
		jsr OP_short_path.be
		#plx_a
		rts

OP_short_path	.proc
		ldx #0
be		lda path_exp,x
		beq ueji
uejilp		lda path_exp+1,x
		sta path_exp,x
		beq ueji
		inx
		bne uejilp
ueji		lda path_exp
		rts
		.pend
;----------------------------------
; set Z=1 if special character found
;
is_special_s	.proc
		cmp #"?"
		beq is_special.x
		cmp #"*"
		beq is_special.x
is_special_p	.proc
		cmp #"/"
		beq is_special.x
is_special	.proc
		cmp #0
		beq x
		cmp #","
		beq x
		cmp #"="
		beq x
		cmp #":"
x		rts
		.pend
;-----------------------------------
;This small routine takes care about that complex wildcard matching...
;
check_wildtype	.proc
		lda #0
		sta wildcard_csy_exp
		lda #direntry_filetype_exp-direntry_name_exp
		gne check_wildcards.wloop2

check_wildcards	.proc
		lda direntry_attrib_exp
		and #FATTR_CLOSED | FTYPE_MASK
		cmp #FTYPE_DIR		;dirlabel as ".." and "."
		bne wilin
		jsr OP_get_current_drive_type
		and #FS_MASK
		eor #FS_CFS		;Virtual "." and ".." only on CFS fs
		bne wilintt
		sta direntry_name_exp+2
		lda #"."
		sta direntry_name_exp
		sta direntry_name_exp+1
		jsr wilin		;check ".."
		bcc foun		;found
		ldx #3			;Current dir in filesize field
svx		lda direntry_size_exp,x
		sta direntry_pointer_exp,x
		dex
		bpl svx
wilintt		lda #0
		sta direntry_name_exp+1	;check "."

wilin		lda #0
		sta wildcard_csy_exp
wloop2		sta wildcard_csx_exp
		tax
		#ldy_l_a wildcard_csy_exp

wloop1		#lda_zp_y current_path
		cmp #"?"
		bne wilany		;C=1
		lda direntry_name_exp,x
		bne wok
foun		rts			;-=> not found (c=1)

wilany		bcs wnspec		;mostly we compare only real text filenames...
		cmp #"*"
		bne wnstar		;C=1
		iny
		#sty_l_a wildcard_csy_exp
		#stx_l_a wildcard_csx_exp
		gcs wloop1

wnstar		jsr is_special_p
		bne wnspec
		lda direntry_name_exp,x
		beq wiqq
		gne wki2

wnspec		cmp direntry_name_exp,x
		bne wki

wok		inx
		iny
		cpx #direntry_filetype_exp-direntry_name_exp+3;exit on end of filetype
		beq gg
		cpx #16			;exit on end of filename
		bne wloop1
gg		#lda_zp_y current_path
		jsr is_special_p
		beq wiqq
		cmp #"*"
		bne wsad
		iny
		gcs gg
wiqq		clc
		rts			;-=> found (c=0)

wki		lda direntry_name_exp,x
		beq wsad
wki2		lda wildcard_csy_exp
		beq wsad
		lda wildcard_csx_exp
		clc
		adc #1
		cmp #direntry_filetype_exp-direntry_name_exp+3;exit on end of filetype
		beq gg
		cmp #16
		bne wloop2		;exit with err if end of name
		geq gg
wsad		sec
		rts			;-=> not found (c=1)
		.pend
;----------------------------------
; Extract open mode from filename
;
OP_path_openmode .proc
		ldx #255
gdii		inx
		lda path_exp,x
		jsr is_special
		bne gdii
		ldy #0
		cmp #","
		beq crt
en		tya
		rts

crt		jsr OP_short_path_here
		lda path_exp,x
		jsr is_special
		beq en
		cpy #0
		bne crt
		tay
		gne crt
		.pend
;----------------------------------
; Extract =xxx or ,xxx from filename
;
OP_path_searchtype .proc
		ldx #255
gdii		inx
		lda path_exp,x
		jsr is_special
		bne gdii
		tya
		ldy #0
		eor path_exp,x
		beq crt
		sec
		rts			;found something other...

crt		jsr OP_short_path_here
		lda path_exp,x
		jsr is_special
		beq en
		cpy #15
		beq crt
		#sta_l_y_ path_searchtype_exp
		iny
		gne crt

en		cpy #1
		bne ks

		lda path_searchtype_exp
		ldy #7*4
fl		cmp ve1-4,y
		beq vn
		dey
		dey
		dey
		dey
		bne fl
		iny
		gne ks

vn		ldx #2
se		lda ve1-1,y
		sta path_searchtype_exp,x
		dey
		dex
		bpl se
		ldy #3

ks		lda #0
		#sta_l_y_x path_searchtype_exp
		clc
		rts			;ok

ve1		.text "D","DEL"
		.text "S","SEQ"
		.text "P","PRG"
		.text "U","USR"
		.text "L","REL"
		.text "C","CBM"
		.text "B","DIR"
		.text "J","LNK"
		.pend

OP_XIT_resolv_path_part .proc
		pha
		jsr OP_get_partnum
		pla
;----------------------------------
; Resolv path including links
;
; Valid path_partition_exp needed!
;
OP_XIT_resolv_path .proc
		sta resolv_mode_exp

		jsr OP_get_current_drive_type
		and #FS_MASK
		cmp #FS_CFS
		beq vw
		lda #DIRMODE_OFS
		.byte SKIP2
vw		lda #DIRMODE_CFS
		sta direntry_mode_exp

		lda #<path_exp
		sta current_path	;for check_wildcard
		lda #>path_exp
		sta current_path+1
		.if SUPERCPU=1
		lda #1
		sta current_path+2
		.fi
		clv
		lda #LINK_DEPTH
		sta resolv_loop_exp

resolv_pathloop	lda #0
		sta resolv_anything_exp
		sec
		#dec_l_a resolv_loop_exp
		beq LWOP_reporterror_39_xit;symlink loop...


		lda path_exp
		eor #"/"
		cmp #1			;C=1 relative path
		php

		lda path_partition_exp
		jsr OP_XIT_check_partition_selection;-=> error
		inx
		inx
		inx

		plp
		ldy #3
bnmf		lda partition_root_dir_exp,x
		bcc iubjb		;C=1 relative path
		bvs resolv_next_dir	;Link file containing relative path...
		lda partition_workdir_exp,x
iubjb		sta next_sector,y
		dex
		dey
		bpl bnmf
		inx
		#stx_l_a partition_currentp_exp

resolv_next_dir	#cnext next_sector , resolv_this_dir_exp

		lda path_exp
skipper		cmp #"/"
		bne wef
		jsr OP_short_path	;skip /, //, /// etc
		jmp skipper
wef
		lda #0
		ldx #4			;no empty entry so far
lw		sta resolv_empty_entry_exp-1,x
		dex
		bne lw
		sta resolv_last_part_exp

jkdj		lda path_exp,x
		jsr is_special
		bne nresend		;last part of filename not resolved
		cpx #0
		beq cdu			;nothing resolvable
		#stx_l_a resolv_last_part_exp

					;found a "xxxx,e" or "xxxx=e" or "xxxx:e" or "xxxx"

		lda resolv_mode_exp
		beq cdu

		and #RESOLV_FILETYPE_MASK
		beq resolv_nxt_sctr	;we resolv everything...
		ldy #","
		jsr OP_path_searchtype
		bcc nsb
		lda resolv_mode_exp
		cmp #RESOLV_LOAD_MODE
		beq in
		cmp #RESOLV_OPEN_MODE
		bne cv
		lda ActualSecAddr
		and #$0f
		eor #1
		cmp #1
		.byte SKIP1		;* for 0,2-14 prg for 1
cv		clc
in		ldx #3
l		lda cprg,x
		bcc tu
		lda cany,x
tu		sta path_searchtype_exp,x
		dex
		bpl l

nsb		lda resolv_mode_exp
		cmp #RESOLV_OPEN_MODE
		bne vwr
		jsr OP_path_openmode
		cmp #"A"		;append
		beq ca
		cmp #"W"		;write
		beq ca
		cmp #"M"		;modify
		beq ca
		lda #"R"		;default
ca		sta resolv_open_mode_exp
vwr		jmp resolv_nxt_sctr
cprg		.text "PRG",0
cany		.text "*",0


cdu		clc
		gcc LWOP_reporterror_ok	;-=> exit 00, OK

nresend		inx
		cmp #"/"
		bne jkdj

					;found a "xxxxxx/"


resolv_nxt_sctr	#cnext next_sector , resolv_this_sector_exp

		jsr XIT_upload_next_sector;-=> error
		ldy #1			;1 sector
		jsr OP_read
		jsr LWOP_reporterror_read_xit;-=> error

		lda #16			;16 directory entrys
		sta resolv_anything_exp
		sta direntry_number_exp
resolv_nxt_ntry	#dec_l_a direntry_number_exp
		bpl resnotlast

		jsr ata_pi_nomore_data
		jsr LWOP_reporterror_read_xit;-=> error

		lda next_sector
		ora next_sector+1
		ora next_sector+2
		ora next_sector+3
		bne resolv_nxt_sctr
		sec			;path not found...
		lda resolv_last_part_exp
		beq LWOP_reporterror_39_xit;-=> path not found
		lda resolv_mode_exp
		and #RESOLV_FILENF_MASK
		beq LWOP_reporterror_62_xit
		rts			;-=> exit, file not found

resnotlast	jsr OP_read_to_direntry

		lda resolv_last_part_exp
		beq np
					;last part of filename
		lda direntry_attrib_exp
		and #FATTR_CLOSED | FTYPE_MASK
		beq res_empt		;empty entry
		gne reso

np		lda direntry_attrib_exp
		and #FATTR_CLOSED | FATTR_EXEC | FTYPE_MASK
		cmp #FATTR_CLOSED | FATTR_EXEC | FTYPE_DIR;Executeable directory?
		beq reso
		cmp #FATTR_EXEC | FTYPE_DIR;Executeable directory? (./..)
		beq reso
		and #FATTR_CLOSED | FTYPE_MASK
		cmp #FATTR_CLOSED | FTYPE_LNK;Closed link?
		beq reso
		gne resolv_nxt_ntry

res_empt	ldx #3			;Remember this empty entry position...
xkd2		lda resolv_this_sector_exp,x
		sta resolv_empty_entry_exp,x
		dex
		bpl xkd2
		gmi resolv_nxt_ntry


reso		jsr check_wildcards
		bcs resolv_nxt_ntry	;no match

		lda resolv_last_part_exp
		beq npjx
		#ldatax resolv_mode_exp
		bpl vmw
		lda direntry_attrib_exp
		and #FATTR_CLOSED | FTYPE_MASK
		cmp #FATTR_CLOSED | FTYPE_LNK
		beq npjx		;link!

vmw		cpx #RESOLV_SAVE_MODE
		beq mds
		cpx #RESOLV_OPEN_MODE
		beq mds
		cpx #RESOLV_LOAD_MODE
		bne mds2
		lda direntry_attrib_exp
		and #FATTR_CLOSED | FATTR_EXEC | FTYPE_MASK
		cmp #FATTR_CLOSED | FATTR_EXEC | FTYPE_NORMAL
		bne resolv_nxt_ntry	;not executeable
mds		lda current_path
		pha
		lda current_path+1
		pha
		lda #<path_searchtype_exp
		sta current_path
		lda #>path_searchtype_exp
		sta current_path+1
		jsr check_wildtype
		pla
		sta current_path+1
		pla
		sta current_path
		bcs resolv_nxt_ntry	;filetype does not match
		gcc npjx

mds2		lda direntry_attrib_exp
		cpx #RESOLV_MD_MODE
		beq vt
		cpx #RESOLV_S_MODE
		beq vt2
		cpx #RESOLV_CD_MODE
		bne npjx
vt		and #FATTR_EXEC | FTYPE_MASK
		cmp #FATTR_EXEC | FTYPE_DIR
		bne resolv_nxt_ntry	;not executeable dir
		geq npjx

vt2		and #FTYPE_MASK
		cmp #FTYPE_DIR
		beq resolv_nxt_ntry	;directory
;                gne npjx

npjx		ldx #31
dfw		lda direntry_exp,x
		sta direntry_f_exp,x
		dex
		bpl dfw
		lda direntry_number_exp
		eor #15
		sta resolv_entry_offs_exp;remember entry position

jek1		#dec_l_a direntry_number_exp;skip rest of sector
		bmi xuxq
		jsr OP_read_to_direntry
		#jmps jek1

xuxq
		jsr ata_pi_nomore_data
		jsr LWOP_reporterror_read_xit;-=> error

		#cnext direntry_f_pointer_exp , next_sector

jkdjb		inx
		lda path_exp,x
		jsr is_special_p
		bne jkdjb
		cmp #"/"
		bne tt

tt		ldy #0
jkdjb2		lda path_exp,x
		#sta_l_y path_exp	;now kill "xxxx/" from "xxxx/sd/df" to get "sd/df"
		beq resk
		inx
		iny
		bne jkdjb2		;should not happen...

resk		lda resolv_last_part_exp
		beq npj
		lda resolv_mode_exp	;we are now in last part
		bpl resolv_next_dir	;don't follow

npj		lda direntry_f_attrib_exp
		and #FATTR_CLOSED | FTYPE_MASK
set_v		cmp #FATTR_CLOSED | FTYPE_LNK
		bne resolv_next_dir

		jsr XIT_insert_link_to_path;-=> error

		jsr OP_get_partnum

		#cnext resolv_this_dir_exp , next_sector
		#sev set_v
		gvs resolv_pathloop

		.pend
;----------------------------------
;Insert a link file content into path
;
XIT_insert_link_to_path .proc
		lda #ram_exp_buffer0
		jsr XIT_read_a_sector	;-=> error

		ldx #0
lp		lda link_data_exp,x
		beq end
		inx
		bne lp
		sec
		gcs LWOP_reporterror_39_xit;-=> Too long path ???

end		ldy #0
lp2		#lda_l_y path_exp
		beq end2
		iny
		sta link_data_exp,x
		inx
		bne lp2
		sec
		gcs LWOP_reporterror_39_xit;-=> buffer overflow

lp3		lda link_data_exp,x
end2		sta path_exp,x
		dex
		cpx #255
		bne lp3

		clc
		rts
		.pend
;----------------------------------
;<partnum>: style
;
OP_XIT_partnum_delimiter .proc
		jsr OP_delimiter_in_path
		bcs OP_get_partnum
		sec
		gcs LWOP_reporterror_30_xit
		.pend
;----------------------------------
;Get partition number from path
;
OP_get_partnum	.proc
		jsr OP_delimiter_in_path
		lda #0
		sta path_partition_exp
		lda path_exp
		bcs kq3			;":" found
		rts

xjke		lda path_partition_exp
		asl
		asl
		adc path_partition_exp
		asl
		adc path_exp
		sec
		sbc #$30
		sta path_partition_exp
		jsr OP_short_path
kq3		cmp #$30		;0
		bcc *+6
		cmp #$3a		;: (10)
		bcc xjke
		beq kq2
		cmp #"/"		;CMD style path?
		bne nocmdp
		jsr OP_short_path
		jsr OP_delimiter_in_path
		#jmps OP_short_path_here;Kill ":" from cmd style path

nocmdp		jsr OP_short_path	;skip stuff between partition number and ":"
		cmp #":"		;Not too optimal way ;)
		bne nocmdp
kq2		#jmps OP_short_path
		.pend
